/****************************************************************
sh_7144.c
Copyright (c) Kosaka Lab CS TNCT
****************************************************************/

#include<stdarg.h>
#include "iodefine.h"
#include "sh_7144.h"

/* ------------------------------------------------- */
/* SCI1 INITIALIZATION fixed baud at 38400           */
/* ------------------------------------------------- */
void initSCI1()
{
	long int i;
	
	MST.CR1.BIT._SCI1 = 0; /*reset SCI1 module standby*/
	
	PFC.PACRL2.BIT.PA4MD = 1;/* PA4MD 0:PA4, 1:TXD1 */
	PFC.PACRL2.BIT.PA3MD = 1;/* PA3MD 0:PA3, 1:RXD1 */
	SCI1.SCR.BYTE = 0;       /* clear all flags */
	SCI1.SCR.BYTE = 0;       /* clear all flags */
	                  /* 2400-38400baud are available at n=0(cks1=0,cks2=0) */
	SCI1.SMR.BYTE = 0;       /* Async, 8bit , NoParity, stop1, 1/1 */
	SCI1.BRR = 19;	         /* 38400baud (CPU=25.0MHz) */
	for(i=0;i<1000000;i++)      /* wait more than 1bit time */
	SCI1.SCR.BYTE = 0x30;    /* scr = 0011 0000 (TE=1,RE=1) */
	return;
}

/* ------------------------------------------------- */
/* GET BYTE FROM SCI1 */
/* ------------------------------------------------- */
short int SCI1_getChar()
/* return value 0x00-0xFF:received data */
/*              -2(0xFFFE):error */
{
	short int flags,recdata;
	do {
		flags = SCI1.SSR.BYTE;
		if (flags&0x38) {/* error */
			SCI1.SSR.BIT.RDRF = 0;
			SCI1.SSR.BIT.ORER = 0;
			SCI1.SSR.BIT.FER = 0;
			SCI1.SSR.BIT.PER = 0;
			return -2;
		}
		if (flags&0x40) {/* normally received one data */
			SCI1.SSR.BIT.RDRF = 0;
			recdata=SCI1.RDR;
			return recdata;
		}
	} while (1);
}

/* ------------------------------------------------- */
/* CHECK SCI BUFFER AND GET DATA */
/* ------------------------------------------------- */
short int SCI1_chkgetChar()
/* return value -1(0xFFFF):no received data  */
/*              0x00-0xFF:received data */
/*              -2(0xFFFE):error */
{
	short int flags,recdata;
	flags = SCI1.SSR.BYTE;
	if (flags&0x38) {/* error */
		SCI1.SSR.BIT.RDRF = 0;
		SCI1.SSR.BIT.ORER = 0;
		SCI1.SSR.BIT.FER = 0;
		SCI1.SSR.BIT.PER = 0;
		return -2;
	}
	if (flags&0x40) {/* normally received one data */
		recdata=SCI1.RDR;
		SCI1.SSR.BIT.RDRF = 0;
		return recdata;
	} else {
		return -1;
	}
}

/*SCI1蕶[return]I[C'\n'͎擾Ȃ*/
/*^HŃobNXyCX*/
int SCI1_getString(char *buff,int max)
{
	int i,ch;
	for (i=0;i<max-1;i++) {
		ch=SCI1_getChar(); /*1擾*/
		*buff=(char)ch; /*1擾*/
		if (*buff=='\r'||ch<0) {
			*buff=0;
			return i+1;
		}
		if (*buff==0x8) {
			buff-=2;
			i-=2;
		}
		if (*buff!='\n') buff++;
		else i--;
	}
	*buff=0;
	return i+1;
}

short int atoi(char *buff)
{
	int x=0,y,m=0,n=0,v=0,i=0;
	y=buff[i];
	while(y!=0){
		if(y=='-') m=1;
		if('a'<=y&&y<='z') y=y-'a'+'A';
		if(y=='0') n=1;

		if(v==1){
			if('0'<=y&&y<='9'){
				y=y-'0';
			}
			else if('A'<=y&&y<='F'){
				y=y-'A'+10;
			}
			x=16*x+y;
		}

		if(n==1&&y=='X'){
			v=1;
		}
				
		if(v==0&&'0'<=y&&y<='9'){
			y=y-'0';
			x=10*x+y;
		}

		y=buff[++i];
	}
	if(m==1) x=-x;
	return x;
}

/*SCI1փvvg\āCSCI1萮l*/
int SCI1_getInt(char prompt[])
/*getting integer from serial port*/
/* format 123[ret] */
/*        -123[ret] */
/*        0x1a[ret] */
/*        -0x100[ret] */
{
	int x=0,y,m=0,n=0,v=0,i=0;
	char buff[16];
	SCI1_putString(prompt);
	SCI1_getString(buff,16);
	return atoi(buff);
}

/* ------------------------------------------------- */
/* PUT BYTE TO SCI1 */
/* ------------------------------------------------- */
void SCI1_putChar(char c)
{
	while(!SCI1.SSR.BIT.TDRE);
	SCI1.TDR = c;
	SCI1.SSR.BIT.TDRE = 0;
	return;
}

void SCI1_putString(char *str)
{
	while(*str){
		SCI1_putChar(*str);
		str++;
	}
}

/************************************************************
Device_printf
************************************************************/

const char hexstring[]="0123456789abcdef0123456789ABCDEF";
#define MAXDIGIT 34
const int SCI1device=0;
const int STRdevice= 1;

static char *currentstrptr;
static void STR_putChar(char ch)
{
	*currentstrptr++=ch;
	*currentstrptr=0;
}

static void STR_putString(char *ptr)
{
	while(*ptr) STR_putChar(*ptr++);
}

void Device_printf(int device, char *format, va_list arg_ptr)
{
	void (*device_putchar)(char ch);
	void (*device_puts)(char *ptr);
	char buf[MAXDIGIT];
	unsigned char flag=0;  /*%d:bit2 l:bit1 %:bit0 */
	unsigned char digit=0; /*  */
	unsigned char minus=0;
	char fill=' ',format1=' ';
	unsigned char radix=10; /*Ni*/
	char sign=' ';
	char *ptr=buf; /*o͕|C^*/
	unsigned char cntr=0; /*o͕JE^*/
	unsigned char shift=0; /*16iVtg 0 or 6*/
	unsigned char i;
	unsigned long int value=0;
	if (device==SCI1device) {
		device_putchar=SCI1_putChar;
		device_puts=SCI1_putString;
	} else { /*device==STRdevice*/
		device_putchar=STR_putChar;
		device_puts=STR_putString;
	}
	/*va_start(arg_ptr,format);*/
	while (*format) {
		format1=*format;
		if (flag==0) {
			if (format1=='%') {
				flag=1;
				digit=0;
				fill=' ';
				minus=0;
				radix=0;
				ptr=&buf[MAXDIGIT-1];
				*ptr--='\0';
				cntr=0;
				shift=0;
				sign='+';
			} else {
				device_putchar(format1);
			}
		} else {
			if (format1=='l') {
				flag|=2;
			} else if ('0'<=(format1)&&(format1)<='9') {
				if (digit==0 && format1=='0') {
					fill='0';
				} else {
					digit=digit*10+((format1)-'0');
					if (MAXDIGIT-2<digit) digit=MAXDIGIT-2;
				}
			} else if (format1=='-') {
				minus=1;
			} else if (format1=='d') {
				flag|=4;
				radix=10;
			} else if (format1=='u') {
				radix=10;
			} else if (format1=='x') {
				radix=16;
			} else if (format1=='X') {
				radix=16;shift=16;
			} else if (format1=='o') {
				radix=8;
			} else if (format1=='b') {
				radix=2;
			} else if (format1=='p') {
				radix=16;shift=16;digit=8;fill='0';flag|=2;
			} else if (format1=='c') {
				device_putchar((unsigned char)(va_arg(arg_ptr,int)));
				flag=0;
			} else if (format1=='s') {
				if (digit) {
					cntr=0;ptr=va_arg(arg_ptr,char *);
					while (ptr[cntr]) cntr++; /*cntr͕*/
					if (!minus) for (i=cntr;i<digit;i++) device_putchar(' ');
					device_puts(ptr);
					if (minus) for (i=cntr;i<digit;i++) device_putchar(' ');
				} else {
					device_puts(va_arg(arg_ptr,char *));
				}
				flag=0;
			} else {
				device_putchar(format1);
				flag=0;
			}
			if (radix) {
				switch (flag&6) {
				case 0: /* unsig int */
					value=(unsigned int)va_arg(arg_ptr,int);
					break;
				case 2: /* unsig long int */
					value=va_arg(arg_ptr,long int);
					break;
				case 4: /* sig int */
					value=(long int)va_arg(arg_ptr,int);
					if ((long int)value<0) {
						value=-(long int)value;
						sign='-';
					}
					break;
				case 6: /* sig long int */
					value=va_arg(arg_ptr,long int);
					if ((long int)value<0) {
						value=-(long int)value;
						sign='-';
					}
					break;
				default:
					break;
				}
				while (value) {
					*ptr--=hexstring[value%radix+shift];
					cntr++;
					value/=radix;
				}
				if (cntr==0) {
					*ptr--='0';
					cntr++;
				}
				if (fill==' ') {
					if (sign=='-') {
						*ptr--='-';
						cntr++;
					}
					if (!minus) for (i=cntr;i<digit;i++) device_putchar(' ');
					device_puts(++ptr);
					if (minus) for (i=cntr;i<digit;i++) device_putchar(' ');
				} else {
					for (;cntr<digit-1;cntr++) *ptr--='0';
					if (sign!='-'&&cntr<digit) *ptr--='0';
					else if (sign=='-') *ptr--='-';
					device_puts(++ptr);
				}
				flag=0;
			}
		}
		format++;
	}
	/*va_end(arg_ptr);*/
}

void SCI1_printf(char *format,...)
{
	va_list arg;
	va_start(arg, format);
	Device_printf(SCI1device, format, arg);
	va_end(arg);
}

void sprintf(char *buff,char *format,...)
{
	va_list arg;
	va_start(arg, format);
	currentstrptr=buff;
	Device_printf(STRdevice, format, arg);
	va_end(arg);
}

/* ------------------------------------------------- */
/*                 LED INITIALIZATION                */
/* ------------------------------------------------- */
/*LEDbitȂ킿|[gEbit0,bit1o͂ɐݒ*/
/*void initLed(void)
{
	PFC.PEIORL.WORD |= 0x0003;
}*/
/*#define initLed() (PFC.PEIORL.WORD |= 0x0003)*/

/* ------------------------------------------------- */
/* LET LED ON */
/* ------------------------------------------------- */
/*number0܂1*/
void turnOnLed(short int number)
{	
	static const unsigned short int mask[]={0x1,0x2};
	PE.DRL.WORD |= mask[number];
}

/* ------------------------------------------------- */
/* LET LED OFF */
/* ------------------------------------------------- */
/*number0܂1*/
void turnOffLed(short int number)
{
	static const unsigned int mask[]={0xfffe,0xfffd};
	PE.DRL.WORD &= mask[number];
}


/* ------------------------------------------------- */
/*              SW INITIALIZATION                    */
/* ------------------------------------------------- */
/*SWPEbit2Ɋ蓖ĂĂ*/
/*void initSW(void)
{
	PFC.PEIORL.WORD &= 0xfffb;
}*/
/*#define initSW() (PFC.PEIORL.WORD &= 0xfffb)*/

/* SW̏ԂԂ */
/*ONȂ1AłȂ0Ԃ*/
short int getSW(void)
{
	return 1-((PE.DRL.WORD>>2)&0x1);
}


/* ------------------------------------------------- */
/*         COMPARE MATCH TIMER INITIALIZATION        */
/* ------------------------------------------------- */
/* RyA}b`^C}ɂ^C}荞݂̏ */
/* 荞݊Ԋu͈peiodŒPʂ̓secł */
/* 0<period<1342156 Ȃ̂ōŒ1.342156sec܂ŉ\ */
/* `lQ̂łch0,1̂ǂ炩I */
/* 荞݃xintLebel1`15ɂ */
void initCMTInt(unsigned long int period,int ch,int intLebel)
{
	/* PӁF25MHz */
	unsigned long int cmcor;
	unsigned char cks;
	if (period<20970) {
		cmcor=(period*25+4)/8;
		cks=0; /* P/8 */
	} else if (period<83883) {
		cmcor=(period*25+16)/32;
		cks=1; /* P/32 */
	} else if (period<335539) {
		cmcor=(period*25+64)/128;
		cks=2; /* P/128 */
	} else {
		cmcor=(period*25+256)/512;
		cks=3; /* P/512 */
	}
	MST.CR2.BIT._CMT=0; /*CMT X^oC*/
	if (ch==0) {
		CMT.CMSTR.BIT.STR0=0; /* RyA}b`^C}0Xgbv */
		CMT0.CMCOR=cmcor-1;
		CMT0.CMCSR.BIT.CKS=cks; /* ӂ̐ݒ */
		CMT0.CMCSR.BIT.CMIE=1; /* 荞݉ݒ */
		INTC.IPRG.BIT._CMT0=intLebel; /* CMT0 荞݃Rg[̊荞݃x */
	} else {
		CMT.CMSTR.BIT.STR1=0; /* RyA}b`^C}1Xgbv*/
		CMT1.CMCOR=cmcor-1;
		CMT1.CMCSR.BIT.CKS=cks; /* ӂ̐ݒ */
		CMT1.CMCSR.BIT.CMIE=1; /* 荞݉ݒ */
		INTC.IPRG.BIT._CMT1=intLebel; /* _CMT1 荞݃Rg[̊荞݃x */
	}
}

/*#define startCMT0()      (CMT.CMSTR.BIT.STR0=1) /* RyA}b`^C}0X^[g */
/*#define stopCMT0()       (CMT.CMSTR.BIT.STR0=0) /* RyA}b`^C}0Xgbv */
/*#define clearCMFlag0()  (CMT0.CMCSR.BIT.CMF=0) /* RyA}b`tbO0NA */
/*#define startCMT1()      (CMT.CMSTR.BIT.STR1=1) /* RyA}b`^C}1X^[g */
/*#define stopCMT1()       (CMT.CMSTR.BIT.STR1=0) /* RyA}b`^C}1Xgbv*/
/*#define clearCMFlag1()  (CMT1.CMCSR.BIT.CMF=0) /* RyA}b`tbO1NA */
